unit RyesapUtils;

//------------------------------------------------------------------------------
// About Ryesap...
//
// author         :  MinMin
// version        :  build 0.0.3
// date           :  Mar, 2004 (since 2002)
// copyright      :  Copyright (C) 2002-2004 MinMin.
// e-mail         :  nmnljam@yahoo.co.jp
// redistribution :  This program is distributed under the GPL. see below.
//------------------------------------------------------------------------------
// About StripReloc...
//
//   This program is FREEWARE;
//   you may redistibute it and/or modify it freely.
//   In case of redistribution, I hope you will write above copyright
//   in addition.
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//------------------------------------------------------------------------------

{
  StripReloc v1.12
  Strip relocation section from Win32 PE files
  Copyright (C) 1999-2004 Jordan Russell. All rights reserved.
  http://www.jrsoftware.org/

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $jrsoftware: stripreloc/StripReloc.dpr,v 1.12 2004/02/24 06:40:04 jr Exp $
}

interface

uses
  Windows,
  Messages;

function FileExistsOnAttr(lpFileName: PChar): boolean;
procedure SetPosition(hWindow: HWND);
procedure RemoveFilesFromListBox(hListBox: HWND);
procedure AddStringToListBox(hListBox: HWND; lpszString: LPCSTR);
function IsPEFileExt(lpFileName: PChar; bDllEnabled: boolean): boolean;
procedure GetFilesOnDir(hListBox: HWND; lpDirectory: PChar; bDllEnabled: boolean);

implementation

function FileExistsOnAttr(lpFileName: PChar): boolean;
var
  dwAttr: DWORD;
begin
  dwAttr := GetFileAttributes(lpFileName);
  Result := (dwAttr <> DWORD(-1)) and ((dwAttr and FILE_ATTRIBUTE_DIRECTORY) = 0);
end;

procedure SetPosition(hWindow: HWND);
var
  zx,
  zy,
  zWidth,
  zHeight : integer;
  Rect: TRect;
begin
  GetWindowRect(hWindow, Rect);
  with Rect do
  begin
    zWidth  := Right  - Left;
    zHeight := Bottom - Top;
    zx      := (GetSystemMetrics(SM_CXSCREEN) - zWidth)  div 2;
    zy      := (GetSystemMetrics(SM_CYSCREEN) - zHeight) div 2;
    MoveWindow(hWindow, zx, zy, zWidth, zHeight, LongBool(true)); 
  end;
end;

procedure RemoveFilesFromListBox(hListBox: HWND);
var
  i: integer;
  nItemCount: integer;
  nItemSelected: integer;
begin
  nItemCount := SendMessage(hListBox, LB_GETCOUNT, 0, 0);
  if (nItemCount = 0) or (nItemCount = LB_ERR) then
    Exit;
  i := nItemCount-1;
  repeat
    nItemSelected := SendMessage(hListBox, LB_GETSEL, i, 0);
    if nItemSelected > 0 then
      SendMessage(hListBox, LB_DELETESTRING, i, 0);
    Dec(i);
  until (i < 0);
end;

procedure AddStringToListBox(hListBox: HWND; lpszString: LPCSTR);
begin
  SendMessage(hListBox, LB_ADDSTRING, WPARAM(0), LPARAM(lpszString));
end;

function IsPEFileExt(lpFileName: PChar; bDllEnabled: boolean): boolean;
var
  nFileNameLength: integer;
  pPos: PChar;
  strExt: string;
begin
  Result := false;
  nFileNameLength := lstrlen(lpFileName);
  if nFileNameLength < 5 then
    Exit;
  pPos := lpFileName;
  Inc(pPos, nFileNameLength-4);
  strExt := PChar(pPos);
  UniqueString(strExt);
  if (strExt = '.EXE') or (strExt = '.exe') then
    Result := true;
  if (bDllEnabled) and ((strExt ='.dll') or (strExt = '.DLL')) then
    Result := true;
end;

procedure GetFilesOnDir(hListBox: HWND;
  lpDirectory: PChar; bDllEnabled: boolean);
var
  bFileFound: boolean;
  IsErr    : integer;
  strDir   : string;
  str, strMask : string;
  HFile    : THandle;
  Fd       : TWin32FindData;
  nDirNameLength: integer;
begin
  strDir := lpDirectory;
  UniqueString(strDir);
  nDirNameLength := Length(strDir);
  if nDirNameLength <= 0 then
    Exit;
  if strDir[nDirNameLength] <> '\' then
    strDir := strDir + '\';
  strMask := strDir + '*.*';
  HFile   := FindFirstFile(PChar(strMask), Fd);
  if HFile <> INVALID_HANDLE_VALUE then
  begin
    str := String(Fd.cFileName);
    if (str <> '.')  and (str <>'..') and
       (Fd.dwFileAttributes and $0000002F {AnyFiles-faDirectory} <> 0) then
    begin
      str := strDir + str;
      if IsPEFileExt(PChar(str), bDllEnabled) then
        AddStringToListBox(hListBox, PChar(str));
    end;
    IsErr := 0;
    try
      while (IsErr <> ERROR_NO_MORE_FILES) do
      begin
        bFileFound := FindNextFile(HFile, Fd);
        if not bFileFound then
          IsErr := GetLastError;
        if (IsErr <> ERROR_NO_MORE_FILES) then
        begin
          str := String(Fd.cFileName);
          if (str <> '.')  and (str <>'..') and
             (Fd.dwFileAttributes and $0000002F {AnyFiles-faDirectory} <> 0) then
          begin
            str := strDir + str;
            if IsPEFileExt(PChar(str), bDllEnabled) then
              AddStringToListBox(hListBox, PChar(str));
          end;
        end;
      end;
    finally
      Windows.FindClose(HFile);
    end;
    HFile := INVALID_HANDLE_VALUE;
  end;
end;

end.
