unit StripReloc;

//------------------------------------------------------------------------------
// About Ryesap...
//
// author         :  MinMin
// version        :  build 0.0.3
// date           :  Mar, 2004 (since 2002)
// copyright      :  Copyright (C) 2002-2004 MinMin.
// e-mail         :  nmnljam@yahoo.co.jp
// redistribution :  This program is distributed under the GPL. see below.
//------------------------------------------------------------------------------
// About StripReloc...
//
//   This program is FREEWARE;
//   you may redistibute it and/or modify it freely.
//   In case of redistribution, I hope you will write above copyright
//   in addition.
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//------------------------------------------------------------------------------

{
  StripReloc v1.12
  Strip relocation section from Win32 PE files
  Copyright (C) 1999-2004 Jordan Russell. All rights reserved.
  http://www.jrsoftware.org/

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $jrsoftware: stripreloc/StripReloc.dpr,v 1.12 2004/02/24 06:40:04 jr Exp $
}

interface

uses
  Windows, RyesapUtils;

const

  fmOpenRead       = $0000;
  fmOpenWrite      = $0001;
  fmOpenReadWrite  = $0002;
  fmShareExclusive = $0010;
  fmShareDenyWrite = $0020;
  fmShareDenyNone  = $0040;

  chrIntOffset : Char = '0';

resourcestring

  CRLF = #13#10;

  RESSTR_J_PROCESSING  = '... ';
  RESSTR_J_IMAGEHLPLOADERR =
   'G[ >> imagehlp.dll [hł܂.'#13#10'`FbNTނ߂ɕKvł.';
  RESSTR_J_IMAGEHLPMAPERR =
   'G[ >> CheckSumMappedFile [hł܂.'#13#10'`FbNTނ߂ɕKvł.';
  RESSTR_J_FILENOTEXISTS = 'G[ >> t@C݂܂.';
  RESSTR_J_NOTEXEFILE = 'EXEt@Cł͂܂ ';
  RESSTR_J_NOTPEFILE  = 'PEt@Cł͂܂ ';
  RESSTR_J_HASSTRIPPED = 'P[V͂łɍ폜Ă܂ ';
  RESSTR_J_SKIPDLL    = 'DLLł܂.';
  RESSTR_J_FILEHANDLEERR = 't@Cł܂ ';
  RESSTR_J_IMAGEBASEADDR = 'C[Wx[XAhX0x400000ȉł.';
  RESSTR_J_CANNOTDELBAK  = 'x >>obNAbvt@C폜ł܂ł : ';
  RESSTR_J_DELETED    = ' 폜.';


  RESSTR_E_PROCESSING = 'Processing... ';
  RESSTR_E_IMAGEHLPLOADERR =
   'Error >> Unable to load imagehlp.dll.'#13#10'It is required for writing checksum.';
  RESSTR_E_IMAGEHLPMAPERR =
   'Error >> Unable to get address of CheckSumMappedFile in imagehlp.dll.'#13#10'It is required for writing checksum.';
  RESSTR_E_FILENOTEXISTS = 'Error >> Cannot find the file. ';
  RESSTR_E_NOTEXEFILE = 'File is not an EXE file. ';
  RESSTR_E_NOTPEFILE  = 'File is not a PE file. ';
  RESSTR_E_HASSTRIPPED = 'Relocations already stripped from file. ';
  RESSTR_E_SKIPDLL    = 'Skipping; cannot strip a DLL.';
  RESSTR_E_FILEHANDLEERR = 'Cannot handle this file. ';
  RESSTR_E_IMAGEBASEADDR = 'the image base address is less than 0x400000.';
  RESSTR_E_CANNOTDELBAK  = 'Warning >> Could not delete backup file : ';
  RESSTR_E_DELETED    = ' deleted.';




  procedure InitResourceString(IsJapanese: boolean);
  function CalcChecksum(const FileHandle: THandle): DWORD;
  function ProcessStrip(const aFileName: string;
    bUseJapanese: boolean;
    bKeepBackups: boolean;
    bWantValidChecksum: boolean;
    bForceStrip: boolean): string;

implementation

var
  UseJapanese: boolean = false;
  KeepBackups: Boolean = true;
  WantValidChecksum: Boolean = false;
  ForceStrip: Boolean = false;

  strResProcessing :string;
  strResChecksumError : string;
  strResChecksumMapError :string;
  strResFileNotExists : string;
  strResNotEXEFile : string;
  strResNotPEFile  : string;
  strResHasStripped: string;
  strResSkipDLL    : string;
  strResFileHandleError: string;
  strResImageBaseAddr: string;
  strResCannotDeleteBak: string;
  strResDeleted : string;



  ImageHlpHandle: THandle;
  CheckSumMappedFile: function(BaseAddress: Pointer; FileLength: DWORD;
    HeaderSum: PDWORD; CheckSum: PDWORD): PImageNtHeaders; stdcall;

procedure InitResourceString(IsJapanese: boolean);
begin
  if IsJapanese then
  begin
    strResProcessing    := RESSTR_J_PROCESSING;
    strResChecksumError := RESSTR_J_IMAGEHLPLOADERR;
    strResChecksumMapError := RESSTR_J_IMAGEHLPMAPERR;
    strResFileNotExists := RESSTR_J_FILENOTEXISTS;
    strResNotEXEFile    := RESSTR_J_NOTEXEFILE;
    strResNotPEFile     := RESSTR_J_NOTPEFILE;
    strResHasStripped   := RESSTR_J_HASSTRIPPED;
    strResSkipDLL       := RESSTR_J_SKIPDLL;
    strResFileHandleError := RESSTR_J_FILEHANDLEERR;
    strResImageBaseAddr := RESSTR_J_IMAGEBASEADDR;
    strResCannotDeleteBak := RESSTR_J_CANNOTDELBAK;
    strResDeleted       := RESSTR_J_DELETED;
  end
  else
  begin
    strResProcessing    := RESSTR_E_PROCESSING;
    strResChecksumError := RESSTR_E_IMAGEHLPLOADERR;
    strResChecksumMapError := RESSTR_E_IMAGEHLPMAPERR;
    strResFileNotExists := RESSTR_E_FILENOTEXISTS;
    strResNotEXEFile    := RESSTR_E_NOTEXEFILE;
    strResNotPEFile     := RESSTR_E_NOTPEFILE;
    strResHasStripped   := RESSTR_E_HASSTRIPPED;
    strResSkipDLL       := RESSTR_E_SKIPDLL;
    strResFileHandleError := RESSTR_E_FILEHANDLEERR;
    strResImageBaseAddr := RESSTR_E_IMAGEBASEADDR;
    strResCannotDeleteBak := RESSTR_E_CANNOTDELBAK;
    strResDeleted       := RESSTR_E_DELETED;

  end;
end;

function CalcChecksum(const FileHandle: THandle): DWORD;
var
  H: THandle;
  M: Pointer;
  OldSum: DWORD;
begin
  M := nil;
  H := CreateFileMapping(FileHandle, nil, PAGE_READONLY, 0, 0, nil);
  if H = 0 then
  begin
    Result := 0;
    Exit;
  end;
  try
    M := MapViewOfFile(H, FILE_MAP_READ, 0, 0, 0);
    CheckSumMappedFile(M, GetFileSize(FileHandle, nil), @OldSum, @Result);
  finally
    if Assigned(M) then
      UnmapViewOfFile(M);
    CloseHandle(H);
  end;
end;

function Strip(const Filename: string): string;
type
  PPESectionHeaderArray = ^TPESectionHeaderArray;
  TPESectionHeaderArray = array[0..$7FFFFFFF div SizeOf(TImageSectionHeader)-1] of TImageSectionHeader;
var
  BackupFilename: String;
  F, F2: File;
  EXESig: Word;
  PEHeaderOffset, PESig: Cardinal;
  PEHeader: TImageFileHeader;
  PEOptHeader: ^TImageOptionalHeader;
  PESectionHeaders: PPESectionHeaderArray;
  BytesLeft, Bytes: Cardinal;
  Buf: array[0..8191] of Byte;
  I: Integer;
  RelocVirtualAddr, RelocPhysOffset, RelocPhysSize: Cardinal;
  OldSize, NewSize, DiffSize: Cardinal;
  TimeStamp: TFileTime;
  strOutput: string;
  szOutput: array[0..127] of char;
begin
  //Result := 'Processing ' + Filename + '...'+CRLF;
  PEOptHeader := nil;
  PESectionHeaders := nil;
  try
    RelocPhysOffset := 0;
    RelocPhysSize := 0;
    BackupFilename := Filename + '.bak';

    //Write(Filename, ': ');
    AssignFile(F, Filename);
    FileMode := fmOpenRead or fmShareDenyWrite;
    Reset(F, 1);
    try
      OldSize := FileSize(F);
      GetFileTime(TFileRec(F).Handle, nil, nil, @TimeStamp);

      BlockRead(F, EXESig, SizeOf(EXESig));
      if EXESig <> $5A4D {'MZ'} then
      begin
        Result := Result + strResNotEXEFile + '(1)';
        Exit;
      end;
      Seek(F, $3C);
      BlockRead(F, PEHeaderOffset, SizeOf(PEHeaderOffset));
      if PEHeaderOffset = 0 then
      begin
        Result := Result + strResNotPEFile + '(1)';
        Exit;
      end;
      Seek(F, PEHeaderOffset);
      BlockRead(F, PESig, SizeOf(PESig));
      if PESig <> $00004550 {'PE'#0#0} then
      begin
        Result := Result + strResNotPEFile + '(2)';
        Exit;
      end;
      BlockRead(F, PEHeader, SizeOf(PEHeader));
      if not ForceStrip and (PEHeader.Characteristics and IMAGE_FILE_DLL <> 0) then
      begin
        Result := Result + strResSkipDLL;
        Exit;
      end;
      if PEHeader.Characteristics and IMAGE_FILE_RELOCS_STRIPPED <> 0 then
      begin
        Result := Result + strResHasStripped + '(1)';
        Exit;
      end;
      PEHeader.Characteristics := PEHeader.Characteristics or IMAGE_FILE_RELOCS_STRIPPED;
      GetMem(PEOptHeader, PEHeader.SizeOfOptionalHeader);
      BlockRead(F, PEOptHeader^, PEHeader.SizeOfOptionalHeader);
      if (PEOptHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress = 0) or
         (PEOptHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size = 0) then
      begin
        Result := Result + strResHasStripped + '(2)';
        Exit;
      end;
      RelocVirtualAddr := PEOptHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress;
      PEOptHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].VirtualAddress := 0;
      PEOptHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BASERELOC].Size := 0;
      if not WantValidChecksum then
        PEOptHeader.CheckSum := 0;
      GetMem(PESectionHeaders, PEHeader.NumberOfSections * SizeOf(TImageSectionHeader));
      BlockRead(F, PESectionHeaders^, PEHeader.NumberOfSections * SizeOf(TImageSectionHeader));
      for I := 0 to PEHeader.NumberOfSections-1 do
        with PESectionHeaders[I] do
          if (VirtualAddress = RelocVirtualAddr) and (SizeOfRawData <> 0) then
          begin
            RelocPhysOffset := PointerToRawData;
            RelocPhysSize := SizeOfRawData;
            SizeOfRawData := 0;
            Break;
          end;
      if RelocPhysOffset = 0 then
      begin
        Result := Result + strResHasStripped + '(3)';
        Exit;
      end;
      if RelocPhysSize = 0 then
      begin
        Result := Result + strResHasStripped + '(4)';
        Exit;
      end;
      for I := 0 to PEHeader.NumberOfSections-1 do
        with PESectionHeaders[I] do
        begin
          if PointerToRawData > RelocPhysOffset then
            Dec(PointerToRawData, RelocPhysSize);
          if PointerToLinenumbers > RelocPhysOffset then
            Dec(PointerToLinenumbers, RelocPhysSize);
          if PointerToRelocations <> 0 then
          begin
            { ^ I don't think this field is ever used in the PE format.
              StripRlc doesn't handle it. }
            Result := Result + strResFileHandleError + '(1)';
            Exit;
          end;
        end;
      if PEOptHeader.ImageBase < $400000 then begin
        Result := Result + strResFileHandleError + '-- ' + strResImageBaseAddr;
        Exit;
      end;
    finally
      CloseFile(F);
    end;
    if FileExistsOnAttr(PChar(BackupFilename)) then
    begin
      if DeleteFile(PChar(BackupFilename)) = false then
        Exit;
    end;
    Rename(F, BackupFilename);
    try
      FileMode := fmOpenRead or fmShareDenyWrite;
      Reset(F, 1);
      try
        AssignFile(F2, Filename);
        FileMode := fmOpenWrite or fmShareExclusive;
        Rewrite(F2, 1);
        try
          BytesLeft := RelocPhysOffset;
          while BytesLeft <> 0 do begin
            Bytes := BytesLeft;
            if Bytes > SizeOf(Buf) then Bytes := SizeOf(Buf);
            BlockRead(F, Buf, Bytes);
            BlockWrite(F2, Buf, Bytes);
            Dec(BytesLeft, Bytes);
          end;
          Seek(F, Cardinal(FilePos(F)) + RelocPhysSize);
          BytesLeft := FileSize(F) - FilePos(F);
          while BytesLeft <> 0 do begin
            Bytes := BytesLeft;
            if Bytes > SizeOf(Buf) then Bytes := SizeOf(Buf);
            BlockRead(F, Buf, Bytes);
            BlockWrite(F2, Buf, Bytes);
            Dec(BytesLeft, Bytes);
          end;
          Seek(F2, PEHeaderOffset + SizeOf(PESig));
          BlockWrite(F2, PEHeader, SizeOf(PEHeader));
          BlockWrite(F2, PEOptHeader^, PEHeader.SizeOfOptionalHeader);
          BlockWrite(F2, PESectionHeaders^, PEHeader.NumberOfSections * SizeOf(TImageSectionHeader));
          if WantValidChecksum then begin
            PEOptHeader.CheckSum := CalcChecksum(TFileRec(F2).Handle);
            { go back and rewrite opt. header with new checksum }
            Seek(F2, PEHeaderOffset + SizeOf(PESig) + SizeOf(PEHeader));
            BlockWrite(F2, PEOptHeader^, PEHeader.SizeOfOptionalHeader);
          end;
          NewSize := FileSize(F2);
          SetFileTime(TFileRec(F2).Handle, nil, nil, @TimeStamp);
        finally
          CloseFile(F2);
        end;
      finally
        CloseFile(F);
      end;
    except
      DeleteFile(PChar(Filename));
      AssignFile(F, BackupFilename);
      Rename(F, Filename);
      raise;
    end;
    FillChar(szOutput, SizeOf(szOutput), 0);
    wvsprintf(szOutput, '%lu byte(s)', addr(OldSize));
    strOutput := string(szOutput);
    Result := Result + CRLF + strOutput + ' -> ';
    FillChar(szOutput, SizeOf(szOutput), 0);
    wvsprintf(szOutput, '%lu byte(s)', addr(NewSize));
    strOutput := string(szOutput);
    Result := Result + strOutput + CRLF;
    DiffSize := OldSize - NewSize;
    FillChar(szOutput, SizeOf(szOutput), 0);
    wvsprintf(szOutput, '%lu byte(s)', addr(DiffSize));
    strOutput := string(szOutput) + strResDeleted;
    Result := Result + strOutput + CRLF;
    if not KeepBackups then
      if not DeleteFile(PChar(BackupFilename)) then
        Result := Result + CRLF + strResCannotDeleteBak + BackupFilename;
  finally
    FreeMem(PESectionHeaders);
    FreeMem(PEOptHeader);
  end;
end;

function ProcessStrip(const aFileName: string;
  bUseJapanese: boolean;
  bKeepBackups: boolean;
  bWantValidChecksum: boolean;
  bForceStrip: boolean): string;
begin
  Result := strResProcessing + aFileName + CRLF;
  UseJapanese := bUseJapanese;
  KeepBackups := bKeepBackups;
  WantValidChecksum := bWantValidChecksum;
  ForceStrip := bForceStrip;

  if WantValidChecksum then
  begin
    ImageHlpHandle := LoadLibrary('imagehlp.dll');
    if ImageHlpHandle = 0 then
    begin
      Result := Result + strResChecksumError;
      Exit;
    end;
    CheckSumMappedFile := GetProcAddress(ImageHlpHandle, 'CheckSumMappedFile');
    if @CheckSumMappedFile = nil then
    begin
      Result := Result + strResChecksumMapError;
      Exit;
    end;
  end;
  if FileExistsOnAttr(PChar(aFileName)) then
    Result := Result + Strip(aFileName) + CRLF
  else
  begin
    Result := Result + strResFileNotExists + CRLF;
  end;
end;

end.
